@@ -80,7 +80,7 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
             QS <- try(as.vector(t(x$alpha)[att] %*% chol2inv(chol(x$va[att,att])) %*% x$alpha[att]), silent=TRUE)
 
             if (inherits(QS, "try-error"))
-               QS <- NA
+               QS <- NA_real_
 
             if (is.element(x$test, c("knha","adhoc","t"))) {
                QS   <- QS / m
@@ -143,14 +143,14 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
                QM <- try(as.vector(t(x$beta)[btt] %*% chol2inv(chol(x$vb[btt,btt])) %*% x$beta[btt]), silent=TRUE)
 
                if (inherits(QM, "try-error"))
-                  QM <- NA
+                  QM <- NA_real_
 
                if (is.element(x$test, c("knha","adhoc","t"))) {
                   QM   <- QM / m
                   QMdf <- c(m, x$QMdf[2])
                   QMp  <- pf(QM, df1=QMdf[1], df2=QMdf[2], lower.tail=FALSE)
                } else {
-                  QMdf <- c(m, NA)
+                  QMdf <- c(m, NA_integer_)
                   QMp  <- pchisq(QM, df=QMdf[1], lower.tail=FALSE)
                }
 
@@ -208,29 +208,29 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
             zval <- c(Za/se)
 
             if (is.element(x$test, c("knha","adhoc","t"))) {
-               pval <- if (x$ddf.alpha > 0) 2*pt(abs(zval), df=x$ddf.alpha, lower.tail=FALSE) else rep(NA,m)
+               pval <- if (x$ddf.alpha > 0) 2*pt(abs(zval), df=x$ddf.alpha, lower.tail=FALSE) else rep(NA_real_,m)
             } else {
                pval <- 2*pnorm(abs(zval), lower.tail=FALSE)
             }
 
             ### omnibus test of all hypotheses (only possible if 'Z' is of full rank)
 
-            QS  <- NA ### need this in case QS cannot be calculated below
-            QSp <- NA ### need this in case QSp cannot be calculated below
+            QS  <- NA_real_ ### need this in case QS cannot be calculated below
+            QSp <- NA_real_ ### need this in case QSp cannot be calculated below
 
             if (rankMatrix(Z) == m) {
 
                QS <- try(as.vector(t(Za) %*% chol2inv(chol(vZa)) %*% Za), silent=TRUE)
 
                if (inherits(QS, "try-error"))
-                  QS <- NA
+                  QS <- NA_real_
 
                if (is.element(x$test, c("knha","adhoc","t"))) {
                   QS   <- QS / m
                   QSdf <- c(m, x$QSdf[2])
-                  QSp  <- if (QSdf[2] > 0) pf(QS, df1=QSdf[1], df2=QSdf[2], lower.tail=FALSE) else NA
+                  QSp  <- if (QSdf[2] > 0) pf(QS, df1=QSdf[1], df2=QSdf[2], lower.tail=FALSE) else NA_real_
                } else {
-                  QSdf <- c(m, NA)
+                  QSdf <- c(m, NA_integer_)
                   QSp  <- pchisq(QS, df=QSdf[1], lower.tail=FALSE)
                }
 
@@ -252,7 +252,7 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
                if (length(unique(rhs)) == 1L) {
                   hyp <- paste0(hyp, " = ", round(rhs, digits=digits[["est"]])) ### add '= rhs' at the right
                } else {
-                  hyp <- paste0(hyp, " = ", .fcf(rhs, digits=digits[["est"]])) ### add '= rhs' at the right
+                  hyp <- paste0(hyp, " = ", fmtx(rhs, digits=digits[["est"]])) ### add '= rhs' at the right
                }
             }
             hyp <- data.frame(hyp, stringsAsFactors=FALSE)
@@ -305,9 +305,9 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
 
                ### omnibus test of all hypotheses (only possible if 'X' is of full rank)
 
-               QM   <- NA ### need this in case QMp cannot be calculated below
-               QMdf <- NA ### need this in case X is not of full rank
-               QMp  <- NA ### need this in case QMp cannot be calculated below
+               QM   <- NA_real_    # need this in case QMp cannot be calculated below
+               QMp  <- NA_real_    # need this in case QMp cannot be calculated below
+               QMdf <- NA_integer_ # need this in case X is not of full rank
 
                if (rankMatrix(X) == m) {
 
@@ -330,14 +330,14 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
                   if (length(x$ddf) == 1L) {
                      ddf <- rep(x$ddf, m)
                   } else {
-                     ddf <- rep(NA, m)
+                     ddf <- rep(NA_integer_, m)
                      for (j in seq_len(m)) {
                         bn0 <- X[j,] != 0
                         ddf[j] <- min(x$ddf[bn0])
                      }
                   }
                } else {
-                  ddf <- rep(NA, m)
+                  ddf <- rep(NA_integer_, m)
                }
 
                ### specification of the right-hand side
@@ -360,16 +360,16 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
                zval <- c(Xb/se)
 
                if (is.element(x$test, c("knha","adhoc","t"))) {
-                  pval <- sapply(seq_along(ddf), function(j) if (ddf[j] > 0) 2*pt(abs(zval[j]), df=ddf[j], lower.tail=FALSE) else NA)
+                  pval <- sapply(seq_along(ddf), function(j) if (ddf[j] > 0) 2*pt(abs(zval[j]), df=ddf[j], lower.tail=FALSE) else NA_real_)
                } else {
                   pval <- 2*pnorm(abs(zval), lower.tail=FALSE)
                }
 
                ### omnibus test of all hypotheses (only possible if 'X' is of full rank)
 
-               QM   <- NA ### need this in case QMp cannot be calculated below
-               QMdf <- NA ### need this in case X is not of full rank
-               QMp  <- NA ### need this in case QMp cannot be calculated below
+               QM   <- NA_real_    # need this in case QMp cannot be calculated below
+               QMp  <- NA_real_    # need this in case QMp cannot be calculated below
+               QMdf <- NA_integer_ # need this in case X is not of full rank
 
                if (rankMatrix(X) == m) {
 
@@ -381,14 +381,14 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
                   QM <- try(as.vector(t(Xb) %*% chol2inv(chol(vXb)) %*% Xb), silent=TRUE)
 
                   if (inherits(QM, "try-error"))
-                     QM <- NA
+                     QM <- NA_real_
 
                   if (is.element(x$test, c("knha","adhoc","t"))) {
                      QM   <- QM / m
                      QMdf <- c(m, min(ddf))
-                     QMp  <- if (QMdf[2] > 0) pf(QM, df1=QMdf[1], df2=QMdf[2], lower.tail=FALSE) else NA
+                     QMp  <- if (QMdf[2] > 0) pf(QM, df1=QMdf[1], df2=QMdf[2], lower.tail=FALSE) else NA_real_
                   } else {
-                     QMdf <- c(m, NA)
+                     QMdf <- c(m, NA_integer_)
                      QMp  <- pchisq(QM, df=QMdf[1], lower.tail=FALSE)
                   }
 
@@ -412,7 +412,7 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
                if (length(unique(rhs)) == 1L) {
                   hyp <- paste0(hyp, " = ", round(rhs, digits=digits[["est"]])) ### add '= rhs' at the right
                } else {
-                  hyp <- paste0(hyp, " = ", .fcf(rhs, digits=digits[["est"]])) ### add '= rhs' at the right
+                  hyp <- paste0(hyp, " = ", fmtx(rhs,  digits=digits[["est"]])) ### add '= rhs' at the right
                }
             }
             hyp <- data.frame(hyp, stringsAsFactors=FALSE)
@@ -440,7 +440,7 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
          stop(mstyle$stop("Method not available for objects of class \"rma.glmm\"."))
 
       if (!identical(class(object), class(object2)))
-         stop(mstyle$stop("Class of 'object1' must be the same as class of 'object2'."))
+         stop(mstyle$stop("Class of 'object' must be the same as class of 'object2'."))
 
       if (!is.null(ddd$test)) {
          test <- match.arg(ddd$test, c("LRT", "Wald"))
@@ -473,14 +473,16 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
       }
 
       ### check if models are based on the same data (TODO: also check for same weights?)
+      ### note: using as.vector() to strip attributes/names, as.matrix() to make both V matrices non-sparse, and
+      ###       isTRUE(all.equal()) because conversion to non-sparse can introduce some negligible discrepancies
 
       if (inherits(object, "rma.uni")) {
-         if (!(identical(as.vector(model.f$yi), as.vector(model.r$yi)) && identical(as.vector(model.f$vi), as.vector(model.r$vi)))) ### as.vector() to strip attributes/names
+         if (!(identical(as.vector(model.f$yi), as.vector(model.r$yi)) && isTRUE(all.equal(as.vector(model.f$vi), as.vector(model.r$vi)))))
             stop(mstyle$stop("Observed outcomes and/or sampling variances not equal in the full and reduced model."))
       }
 
       if (inherits(object, "rma.mv")) {
-         if (!(identical(as.vector(model.f$yi), as.vector(model.r$yi)) && identical(as.matrix(model.f$V), as.matrix(model.r$V)))) ### as.vector() to strip attributes/names, as.matrix() to make both V matrices non-sparse
+         if (!(identical(as.vector(model.f$yi), as.vector(model.r$yi)) && isTRUE(all.equal(as.matrix(model.f$V), as.matrix(model.r$V)))))
             stop(mstyle$stop("Observed outcomes and/or sampling variances/covariances not equal in the full and reduced model."))
       }
 
@@ -508,10 +510,23 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
       if (test == "LRT" && model.f$method == "REML" && (!identical(model.f$X, model.r$X))) {
          if (refit) {
             #message(mstyle$message("Refitting models with ML (instead of REML) estimation ..."))
-            model.f <- try(update(model.f, method="ML"), silent=TRUE)
+            if (inherits(model.f, "rma.uni") && model.f$model == "rma.uni") {
+               #model.f <- try(update(model.f, method="ML", data=model.f$data), silent=TRUE)
+               args <- list(yi=model.f$yi, vi=model.f$vi, weights=model.f$weights, mods=model.f$X, intercept=FALSE, method="ML", weighted=model.f$weighted, test=model.f$test, level=model.f$level, tau2=ifelse(model.f$tau2.fix, model.f$tau2, NA), control=model.f$control, skipr2=TRUE)
+               model.f <- try(suppressWarnings(.do.call(rma.uni, args)), silent=TRUE)
+            } else {
+               # note: this fails when building the docs with pkgdown; not sure why; the approach above at least works for 'rma.uni' objects and is more efficient as it skips the R^2 calculation
+               model.f <- try(update(model.f, method="ML"), silent=TRUE)
+            }
             if (inherits(model.f, "try-error"))
                stop(mstyle$stop("Refitting the full model with ML estimation failed."))
-            model.r <- try(update(model.r, method="ML"), silent=TRUE)
+            if (inherits(model.r, "rma.uni") && model.r$model == "rma.uni") {
+               #model.r <- try(update(model.r, method="ML", data=model.r$data), silent=TRUE)
+               args <- list(yi=model.r$yi, vi=model.r$vi, weights=model.r$weights, mods=model.r$X, intercept=FALSE, method="ML", weighted=model.r$weighted, test=model.r$test, level=model.r$level, tau2=ifelse(model.r$tau2.fix, model.r$tau2, NA), control=model.r$control, skipr2=TRUE)
+               model.r <- try(suppressWarnings(.do.call(rma.uni, args)), silent=TRUE)
+            } else {
+               model.r <- try(update(model.r, method="ML"), silent=TRUE)
+            }
             if (inherits(model.r, "try-error"))
                stop(mstyle$stop("Refitting the reduced model with ML estimation failed."))
             parms.f <- model.f$parms
@@ -562,7 +577,7 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
             R2 <- 100 * max(0, (model.r$tau2 - model.f$tau2) / model.r$tau2)
          }
       } else {
-         R2 <- NA
+         R2 <- NA_real_
       }
 
       ### for 'rma.uni' objects, extract tau^2 estimates
@@ -571,8 +586,8 @@ anova.rma <- function(object, object2, btt, X, att, Z, rhs, digits, refit=FALSE,
          tau2.f <- model.f$tau2
          tau2.r <- model.r$tau2
       } else {
-         tau2.f <- NA
-         tau2.r <- NA
+         tau2.f <- NA_real_
+         tau2.r <- NA_real_
       }
 
       if (test == "LRT") {
