@@ -1,4 +1,4 @@
-plot.profile.rma <- function(x, xlim, ylim, pch=19, xlab, ylab, main, cline=FALSE, ...) {
+plot.profile.rma <- function(x, xlim, ylim, pch=19, xlab, ylab, main, refline=TRUE, cline=FALSE, ...) {
 
    #########################################################################
 
@@ -7,8 +7,8 @@ plot.profile.rma <- function(x, xlim, ylim, pch=19, xlab, ylab, main, cline=FALS
    .chkclass(class(x), must="profile.rma")
 
    if (dev.cur() == 1) {
-      par(mfrow=c(x$comps, 1))
-      #on.exit(par(mfrow=c(1,1)))
+      par(mfrow=n2mfrow(x$comps))
+      #on.exit(par(mfrow=c(1,1)), add=TRUE)
    }
 
    missing.xlim <- missing(xlim)
@@ -19,7 +19,8 @@ plot.profile.rma <- function(x, xlim, ylim, pch=19, xlab, ylab, main, cline=FALS
 
    ### filter out some arguments for the plot() function
 
-   lplot <- function(..., time, LB, startmethod, sub1) plot(...)
+   lplot   <- function(..., time, LB, startmethod, sub1) plot(...)
+   lpoints <- function(..., time, LB, startmethod, sub1, log) points(...)
 
    #########################################################################
 
@@ -46,13 +47,18 @@ plot.profile.rma <- function(x, xlim, ylim, pch=19, xlab, ylab, main, cline=FALS
          x[[2]] <- c(x[[2]][seq_len(pos-1)], x$maxll, x[[2]][pos:length(x[[2]])])
       }
 
-      lplot(x[[1]], x[[2]], type="o", xlab=xlab, ylab=ylab, main=main, bty="l", pch=pch, xlim=xlim, ylim=ylim, ...)
-      abline(v=x$vc, lty="dotted")
-      abline(h=x$maxll, lty="dotted")
+      lplot(x[[1]], x[[2]], type="n", xlab=xlab, ylab=ylab, main=main, bty="l", xlim=xlim, ylim=ylim, ...)
+
+      if (refline) {
+         abline(v=x$vc, lty="dotted")
+         abline(h=x$maxll, lty="dotted")
+      }
 
       if (cline)
          abline(h=x$maxll - qchisq(0.95, df=1)/2, lty="dotted")
 
+      lpoints(x[[1]], x[[2]], type="o", pch=pch, ...)
+
    } else {
 
       for (j in seq_len(x$comps)) {
